package Test::Repocop::ACLWrapper;
use strict;
use warnings;
use Test::Repocop::Log;
use Test::Repocop::ExternalACL;
use Test::Repocop::ALTLinuxACL;
use Source::Shared::CLI;
use Test::Repocop::CLI;
use Test::Repocop::CLI::Container;
our @ISA=qw/Source::Shared::CLI/;
push @Test::Repocop::CLI::Container::OPT_CLASSES, __PACKAGE__;

our $aclfile;
my $use_altlinux_acl;

our @LONGOPT=(
    "repocop-acl-file=s" => \$aclfile,
    "repocop-acl-altlinux" => \$use_altlinux_acl,
    );
#our @SHORT_OPTIONS=(
#    "acl-file=s" => \$aclfile,
#    "acl-altlinux" => \$use_altlinux_acl,
#    );
#push @Test::Repocop::CLI::LONGOPT, @SHORT_OPTIONS;

eval {
    require ALTLinux::ACL;
    no warnings;
    push @LONGOPT, @ALTLinux::ACL::LONGOPT;
};
my $no_altlinux_acl=$@;

sub print_longopt {
    print "  Repocop acl options:
	--repocop-acl-file <path> Path to generic acl file
	--repocop-acl-altlinux	  Use ALTLinux::ACL as acl provider.
";
    &ALTLinux::ACL::print_longopt() unless $no_altlinux_acl;
}

sub new {
    my $class=shift;
    return Test::Repocop::ExternalACL->new($aclfile) if $aclfile;
    if ($use_altlinux_acl) {
	if ($no_altlinux_acl) {
	    die "Loading ALTLinux ACL failed: $no_altlinux_acl";
	} else {
	    return Test::Repocop::ALTLinuxACL->new(ALTLinux::ACL->new);
	} 
    }
    &repocop_note("Test::Repocop::ACLWrapper: no ACL provider selected.");
    return;
}

1;

=head1	NAME

Test::Repocop::ACLWrapper - repocop library to provide package ACL.

=head1	DESCRIPTION

TODO

=head1	AUTHOR

Written by Igor Vlasenko <viy@altlinux.org>.

=head1	ACKNOWLEGEMENTS

To Alexey Torbin <at@altlinux.org>, whose qa-robot package
had a strong influence on repocop.

=head1	COPYING

Copyright (c) 2008-2022 Igor Vlasenko, ALT Linux Team.

This is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

=cut

