package Test::Repocop::CLI::Datadir;

use strict;
use warnings;
use Cwd;
use Source::Shared::CLI;
use Test::Repocop::Common;
use Test::Repocop::CLI::Container;
use Test::Repocop::CLI;
our $VERSION=0.002;

our @ISA=qw/Source::Shared::CLI/;
push @Test::Repocop::CLI::Container::OPT_CLASSES, __PACKAGE__;

my @pkgtests_dirarg;
my @srctests_dirarg;
my @pkgcollectors_dirarg;
my @srccollectors_dirarg;

my @test_exclude_arg;
my @test_include_arg;

sub print_longopt {
    print "  Repocop datadir options:
	--repocop-pkgtests-dir <path>	Path to pkgtests
	--repocop-srctests-dir <path>	Path to srctests
	--repocop-pkgcollectors-dir <path>	Path to pkgcollectors
	--repocop-srccollectors-dir <path>	Path to srccollectors
	--repocop-exclude-test <test1,...>	exclude tests
	--repocop-include-test <test1,...>	explicitly include listed tests
";
}

our @LONGOPT=(
	"repocop-pkgtests-dir=s"  => \@pkgtests_dirarg,
	"repocop-srctests-dir=s"  => \@srctests_dirarg,
	"repocop-pkgcollectors-dir=s"  => \@pkgcollectors_dirarg,
	"repocop-srccollectors-dir=s"  => \@srccollectors_dirarg,
	"repocop-exclude-test=s"  => \@test_exclude_arg,
	"repocop-include-test=s"  => \@test_include_arg,
    );

our @SHORT_OPTIONS=(
	"pkgtests-dir=s"  => \@pkgtests_dirarg,
	"srctests-dir=s"  => \@srctests_dirarg,
	"pkgcollectors-dir=s"  => \@pkgcollectors_dirarg,
	"srccollectors-dir=s"  => \@srccollectors_dirarg,
	"exclude-test=s"  => \@test_exclude_arg,
	"include-test=s"  => \@test_include_arg,
    );

push @Test::Repocop::CLI::LONGOPT, @SHORT_OPTIONS;

my $rundir=Cwd::getcwd();

sub process_options {
    my @pkgtests_dir=grep {$_ and -d $_} glob ("/usr/{share,lib,lib64}/repocop/pkgtests");
    my @srctests_dir=grep {$_ and -d $_} glob ("/usr/{share,lib,lib64}/repocop/srctests");
    my @pkgcollectors_dir=grep {$_ and -d $_} glob ("/usr/{share,lib,lib64}/repocop/pkgcollectors");
    my @srccollectors_dir=grep {$_ and -d $_} glob ("/usr/{share,lib,lib64}/repocop/srccollectors");

    push @pkgtests_dir, &_filtertestsdir(@pkgtests_dirarg);
    push @srctests_dir, &_filtertestsdir(@srctests_dirarg);
    push @pkgcollectors_dir, &_filtertestsdir(@pkgcollectors_dirarg);
    push @srccollectors_dir, &_filtertestsdir(@srccollectors_dirarg);
    @Repocop::datadir::tests_dir=(@pkgcollectors_dir,@srccollectors_dir,@pkgtests_dir,@srctests_dir);
    @Repocop::datadir::pkgscripts_dir=(@pkgcollectors_dir,@pkgtests_dir);
    @Repocop::datadir::srcscripts_dir=(@srccollectors_dir,@srctests_dir);

    %Repocop::datadir::_EXCLUDE_TEST=map {$_=>1} map {split ','} @test_exclude_arg;
    %Repocop::datadir::_INCLUDE_TEST=map {$_=>1} map {split ','} @test_include_arg;

    die "--include-test should always contain rpm" if %Repocop::datadir::_INCLUDE_TEST
	and not $Repocop::datadir::_INCLUDE_TEST{'rpm'};
}

sub _filtertestsdir {
    my @retval;
    foreach my $dirarg (@_) {
	foreach (split (/[:,]/,$dirarg)) {
	    next unless $_;
	    $_=$rundir."/".$_ unless m!^/!;
	    if (-d $_) {
		push @retval, $_;
	    } else {
		warn "warning: argument to --{pkg,src}tests-dir: $_ is not a directory!\n";
	    }
	}
    }
    return @retval;
}

package Repocop::datadir;

use strict;
use warnings;

our (@tests_dir, @pkgscripts_dir, @srcscripts_dir);

# private; use glob_tests
our %_EXCLUDE_TEST;
our %_INCLUDE_TEST;

use File::Basename; # for glob_tests
sub glob_tests {
    my ($pattern, @dirs)=@_;
    my @inlist=&Test::Repocop::Common::glob_tests($pattern, @dirs);
    return @inlist if not %_EXCLUDE_TEST and not %_INCLUDE_TEST;
    my @outlist;
    my $testname;
    foreach my $intest (@inlist) {
	if ($pattern eq '') {
	    $testname=basename($intest);
	} else {
	    $testname=&Test::Repocop::Common::filepath2testname($intest);
	}
	next if %_EXCLUDE_TEST and $_EXCLUDE_TEST{$testname};
	next if %_INCLUDE_TEST and not $_INCLUDE_TEST{$testname};
	push @outlist, $intest;
    }
    return @outlist;
}

1;
