package Test::Repocop::CLI::IN::PkgId;

use strict;
use warnings;
use File::Basename;
use Test::Repocop::PkgId;
use Test::Repocop::Common;
use Test::Repocop::Workdir; # for --by-srpm-name
use Test::Repocop::TestDB::Filter;
use Source::Shared::CLI;
use Test::Repocop::CLI;
use Test::Repocop::CLI::Container;

our $VERSION=0.001;

our @ISA=qw/Source::Shared::CLI/;
#push @Source::Shared::VERBOSE_CLASSES, __PACKAGE__;
push @Test::Repocop::CLI::Container::OPT_CLASSES, __PACKAGE__;

my ($repocop_arg_lastrun, $repocop_arg_by_src_name);

sub print_longopt {
    print "
  Repocop arguments to ids options:
	--repocop-id-except 	<except> mode. Process all ids except given.
	--repocop-id-given 	<given> mode. Process given ids.
	--repocop-id-last-run 	Use ids from the last repocop collect.
	--repocop-id-by-src-name Assume input to be src names, no repocop ids.
";
}

our @LONGOPT=(
	"repocop-id-except"  => \$Repocop::arg::except,
	"repocop-id-given"  => sub { $Repocop::arg::except=0 },
	"repocop-id-last-run|repocop-id-lastrun"  => \$repocop_arg_lastrun,
	"repocop-id-by-src-name!"  => \$repocop_arg_by_src_name,
    );

our @SHORT_OPTIONS=(
	# public
	"id-except|except"  => \$Repocop::arg::except,
	"id-given|given"  => sub { $Repocop::arg::except=0 },
	"id-last-run|last-run|lastrun"  => \$repocop_arg_lastrun,
	"id-by-src-name|by-src-name!"  => \$repocop_arg_by_src_name,
    );

push @Test::Repocop::CLI::LONGOPT, @SHORT_OPTIONS;

sub process_options {
    &Test::Repocop::Workdir::die_if_nothing_to_report();
}

sub process_arguments {
    if ($repocop_arg_lastrun) {
	my $repocop_lastrun_file="$repocop_metadatadir/last-run";
	die "last run results not found. internal error?\n" unless -e $repocop_lastrun_file;
	my $f;
	open METADATA, $repocop_lastrun_file || die $!;
	while ($f=<METADATA>) {
	    chomp $f;
	    $Repocop::arg::pkgtable{$f}=1 if $f;
	}
	close METADATA;
    }
    ## developed for purge
    if ($repocop_arg_by_src_name) {
	require Test::Repocop::RPMdb;
	# note: too much prepare inside;
	my $dbh=&Test::Repocop::SQL::connect_db("$repocop_test_dbdir/rpm.db");
	my ($srcids, $pkgids)=&Test::Repocop::RPMdb::get_rpm_db_ids_by_srpm_name($dbh,\@ARGV);
	map {$Repocop::arg::pkgtable{$_}=1} @$srcids, @$pkgids;
	&Test::Repocop::SQL::disconnect_db($dbh);
    } else {
	foreach my $rpmarg (@ARGV) {
	    if ( -d "$rpmarg") {
		open (RPMARGS_FILES, "ls -1 $rpmarg |grep -E -x '[^.].*[.]rpm\$' |") || die $!;
		my $f;
		while ($f=<RPMARGS_FILES>) {
		    chomp $f;
		    &_add_file_to_repocop_arg_pkg("$rpmarg/$f") if -e "$rpmarg/$f";
		}
		close RPMARGS_FILES;
	    } elsif (-e $rpmarg) {
		&_add_file_to_repocop_arg_pkg($rpmarg);
	    } elsif (&Test::Repocop::PkgId::is_pkgid($rpmarg)) {
		$Repocop::arg::pkgtable{$rpmarg}=1;
	    } else {
		warn "warning: invalid argument: $rpmarg\n";
	    }
	}
    }
}

sub _add_file_to_repocop_arg_pkg {
    $Repocop::arg::pkgtable{&Test::Repocop::PkgId::get_pkgid_from_rpm_file($_[0])}=1;
}

1;
