package Test::Repocop::CLI::Report;

use strict;
use warnings;
use File::Basename;
use Test::Repocop::Common;
#use Test::Repocop::Workdir; # for --by-srpm-name
use Test::Repocop::TestDB::Filter;
use Source::Shared::CLI;
use Test::Repocop::CLI;
use Test::Repocop::CLI::Container;
# --include-
use Test::Repocop::CLI::Datadir;

our $VERSION=0.003;

our @ISA=qw/Source::Shared::CLI/;
#push @Source::Shared::VERBOSE_CLASSES, __PACKAGE__;
push @Test::Repocop::CLI::Container::OPT_CLASSES, __PACKAGE__;

my (@excludetest, @includetest);
my ($opt_include_distrotests);

sub print_longopt {
    print "
  Repocop report options:
	--repocop-report <level>	Filter by level. Default is $Repocop::arg::reportlevel.
	--repocop-report-exclude-test <test,...>	Filter by test
	--repocop-report-include-test <test,...>	Filter by test
	--repocop-report-include-distrotests include all distrotests in test filter
";
}

our @LONGOPT=(
    	# embedded
	"repocop-report|repocop-report-level=s" => \$Repocop::arg::reportlevel,
	"repocop-report-et|repocop-report-exclude-test=s"  => \@excludetest,
	"repocop-report-it|repocop-report-include-test=s"  => \@includetest,
	"repocop-report-include-distrotests" => \$opt_include_distrotests,
    );

our @SHORT_OPTIONS=(
	# public
	"report-level|report=s" => \$Repocop::arg::reportlevel,
	"report-et|report-exclude-test=s"  => \@excludetest,
	"report-it|report-include-test=s"  => \@includetest,
	"report-include-distrotests" => \$opt_include_distrotests,
    );

push @Test::Repocop::CLI::LONGOPT, @SHORT_OPTIONS;

sub process_options {
    &Test::Repocop::Workdir::die_if_nothing_to_report();
    %Repocop::arg::EXCLUDETEST=map {$_=>1} map {split ','} @excludetest;
    %Repocop::arg::INCLUDETEST=map {$_=>1} map {split ','} @includetest;

    if ($opt_include_distrotests) {
	my @tests_dir=@Repocop::datadir::tests_dir;
	die "Internal error: empty tests_dir" unless @tests_dir;
	%Repocop::arg::INCLUDETEST= map {$_=>1} map {basename(dirname($_))}
	    &Test::Repocop::Common::glob_tests('distrotest', @tests_dir);
    }

    &Repocop::arg::__set_report_mask($Repocop::arg::reportlevel);

}

1;
