package Test::Repocop::Common;

use strict;
use warnings;
use File::Basename;
use File::Path qw(make_path remove_tree);
use Test::Repocop::Workdir qw/$repocop_workdir $repocop_test_dbdir $repocop_testcachedir/;

my $repocop_test_tmpdirname='tmp';

sub set_test_environment {
    my ($testname)=@_;
    $ENV{'REPOCOP_STATEDIR'}=$repocop_testcachedir;
    $ENV{'REPOCOP_TEST_STATEDIR'}="$repocop_testcachedir/$testname";
    $ENV{'REPOCOP_TEST_TMPDIR'}="$repocop_workdir/$repocop_test_tmpdirname/$testname";
    $ENV{'REPOCOP_TEST_DBDIR'}=$repocop_test_dbdir;
    $ENV{'REPOCOP_TEST_NAME'}=$testname;
    my $repocop_test_db="$repocop_test_dbdir/$testname.db";
    $ENV{'REPOCOP_TEST_DB'}=$repocop_test_db if -e $repocop_test_db;

    # hack; was designed to allow pluging-in external dbs
    # not used; remove me after repocop-report-distrodb > 0.425
    $ENV{'REPOCOP_DISTROTEST_DBDIR'}=$repocop_test_dbdir unless $ENV{'REPOCOP_DISTROTEST_DBDIR'};
}

sub mkdir_test_environment {
    my ($testname)=@_;
    &set_test_environment($testname);
    make_path($ENV{'REPOCOP_TEST_STATEDIR'}, $ENV{'REPOCOP_TEST_TMPDIR'});
}

sub cleanup_test_environment {
    my ($testname)=@_;
    remove_tree("$repocop_workdir/$repocop_test_tmpdirname/$testname");
    # only empty directories are deleted
    rmdir "$repocop_testcachedir/$testname";
}

sub cleanup_test_environments {
    remove_tree("$repocop_workdir/$repocop_test_tmpdirname");
    foreach my $testname (@_) {
	# only empty directories are deleted
	rmdir "$repocop_testcachedir/$testname";
    }
}

sub discard_test_environment {
    my ($testname)=@_;
    remove_tree("$repocop_workdir/$repocop_test_tmpdirname/$testname");
    remove_tree("$repocop_testcachedir/$testname");
}

sub glob_tests {
    my ($file,@dirs)=@_;
    my @retval;
    foreach (@dirs) {
	push @retval, glob "$_/*/$file";
    }
    return @retval;
}

sub filepath2testname {
    return basename(dirname($_[0]));
}

sub find_test_dir {
    my ($testname)=@_;
    my @tests_dir=grep {$_ and -d $_} glob ("/usr/{share,lib,lib64}/repocop/{pkgtests,srctests,pkgcollectors,srccollectors}");
    foreach my $dir(@tests_dir) {
	$dir .= '/'.$testname;
	return $dir if -d $dir;
    }
}

1;

=head1	NAME

Test::Repocop::Common - common private subroutines for repocop.

=head1	DESCRIPTION

Common private subroutines for repocop.
The interface is subject to change without notice.

=head1	AUTHOR

Written by Igor Vlasenko <viy@altlinux.org>.

=head1	ACKNOWLEGEMENTS

To Alexey Torbin <at@altlinux.org>, whose qa-robot package
had a strong influence on repocop.

=head1	COPYING

Copyright (c) 2008-2022 Igor Vlasenko, ALT Linux Team.

This is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

=cut

