package Test::Repocop::ExternalACL;
use strict;
use warnings;

our %PKGNAME2ACL;

sub new {
    my $class=shift;
    my $self = {};
    my $acllist=shift;
    bless $self, $class;
    $self->_load($acllist) unless scalar %PKGNAME2ACL;
    return $self;
}

sub name2acl {
    my ($self,$key)=@_;
    my $ptr=$PKGNAME2ACL{$key};
    return unless $ptr;
    return @{$ptr};
}

########## PRIVATE #################

sub _load {
    my ($self,$acllist)=@_;
    open (ACL,'<',$acllist) || die "can't open acl file $acllist: $!";
    while (<ACL>) {
	chomp;
	if (/([^\t]+)\t(.+)/) {
	    my $name=$1;
	    my @acl=split (/\s+/,$2);
	    $PKGNAME2ACL{$name}=\@acl if @acl;
	} else {
	    warn "acl: strange string $_";
	}
    }
    close(ACL);
}


1;

=head1	NAME

Test::Repocop::ExternalACL - repocop private library for an external ACL file.

=head1	DESCRIPTION

TODO

=head1	ACL File FORMAT

External Acl file format is follows: 

<pkg name><tab><leader>[<tab><other acl member>]...
...

=head1	AUTHOR

Written by Igor Vlasenko <viy@altlinux.org>.

=head1	ACKNOWLEGEMENTS

To Alexey Torbin <at@altlinux.org>, whose qa-robot package
had a strong influence on repocop.

=head1	COPYING

Copyright (c) 2008-2022 Igor Vlasenko, ALT Linux Team.

This is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

=cut

