package Test::Repocop::ExternalTest;

use File::Path qw(make_path);

sub test {
    my $opts=shift;
    die "first arg should be an array reference" unless ref ($opts) eq 'ARRAY';
    my $opt={
	'-test'  => $ENV{'REPOCOP_TEST_NAME'},
	'-pkgkey'=> $ENV{'REPOCOP_PKG_KEY'},
	'-status'=> undef,
	@$opts
    };
    my $pkgkey=$opt->{'-pkgkey'};
    my $testname=$opt->{'-test'};
    my $status=$opt->{'-status'};
    die '-pkgkey option is not found!' unless $pkgkey;
    die '-testname option is not found!' unless $testname;
    die '-status option is required!' unless $status;
    die 'REPOCOP_TEST_OUTPUTDIR environment variable is not set!' unless $ENV{REPOCOP_TEST_OUTPUTDIR};
    my $repocop_test_resultsdir="$ENV{REPOCOP_TEST_OUTPUTDIR}/".substr($pkgkey,0,1)."/$pkgkey";
    make_path($repocop_test_resultsdir);
    my $repocop_test_msgfile="$repocop_test_resultsdir/".$testname.'+++'.$status;
    open(my $fh, '>>', $repocop_test_msgfile) or die "Test::Repocop::ExternalTest::test: can't open $repocop_test_result: $!";
    print $fh @_ if (@_ and $status ne 'skip' and $status ne 'ok');
    close ($fh);
}


sub collect {
    my $opts=shift;
    die "first arg should be an array reference" unless ref ($opts) eq 'ARRAY';
    my $opt={
	'-test'  => $ENV{'REPOCOP_TEST_NAME'},
	'-pkgkey'=> $ENV{'REPOCOP_PKG_KEY'},
	'-entryname'=> 'default',
	@$opts
    };
    my $pkgkey=$opt->{'-pkgkey'};
    my $testname=$opt->{'-test'};
    my $entryname=$opt->{'-entryname'};
    die '-pkgkey option is not found!' unless $pkgkey;
    die '-testname option is not found!' unless $testname;
    die 'REPOCOP_STATEDIR environment variable is not set!' unless $ENV{REPOCOP_STATEDIR};
    my $repocop_testdb="$ENV{REPOCOP_STATEDIR}/$testname/$pkgkey";
    make_path($repocop_testdb);
    my $repocop_collect_entry="$repocop_testdb/$entryname";
    open(my $fh, '>>', $repocop_collect_entry) or die "Test::Repocop::ExternalTest::collect_file: can't open $repocop_collect_entry: $!";
    print $fh @_ if @_;
    close ($fh);
}


1;

=head1	NAME

Test::Repocop::ExternalTest - external repocop perl interface.

=head1	DESCRIPTION

Test::Repocop::ExternalTest - external repocop perl interface.
it is equivalent to repocop-test-* utility call.

usage:
use Test::Repocop::ExternalTest;
&Test::Repocop::ExternalTest::test(
    [-status=>'warn',-pkgkey=>'foo-1.0-alt1',-test=>'some test'],
    'message',...
);

=head1	AUTHOR

Written by Igor Vlasenko <viy@altlinux.org>.

=head1	ACKNOWLEGEMENTS

To Alexey Torbin <at@altlinux.org>, whose qa-robot package
had a strong influence on repocop.

=head1	COPYING

Copyright (c) 2008-2022 Igor Vlasenko, ALT Linux Team.

This is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

=cut

