package Test::Repocop::Metadata;
use strict;
use warnings;
use Carp;
use Test::Repocop::SQL;
use Test::Repocop::Workdir; ## for variables ($repocop_test_dbdir);

my $dbh;

our %PKGID2NICK;
our %PKGID2NAME;
our %PKGID2VERSION;
our %PKGID2RELEASE;
our %PKGID2ARCH;
our %PKGID2PACKAGER;
our %PKGID2SOURCEID;
our $sourceid2specfilename;

sub new {
    my $class=shift;
    my $self = {};
    bless $self, $class;
    $self->_load() unless scalar %PKGID2NAME;
    return $self;
}

sub nick {
    my ($self,$key)=@_;
    return $PKGID2NICK{$key};
}

sub name {
    my ($self,$key)=@_;
    if (!defined($key)) {
	carp 'Test::Repocop::Metadata: name: got undef';
    }
    return $PKGID2NAME{$key};
}

sub version {
    my ($self,$key)=@_;
    return $PKGID2VERSION{$key};
}

sub release {
    my ($self,$key)=@_;
    return $PKGID2RELEASE{$key};
}

sub arch {
    my ($self,$key)=@_;
    return $PKGID2ARCH{$key};
}

sub packager {
    my ($self,$key)=@_;
    return $PKGID2PACKAGER{$key};
}

sub sourceid {
    my ($self,$key)=@_;
    return $PKGID2SOURCEID{$key};
}

######### protected ################

sub sourceid2specfilename {
    $sourceid2specfilename=Test::Repocop::Metadata::__sourceid2specfilename() unless $sourceid2specfilename;
    return $sourceid2specfilename;
}

########## PRIVATE #################

sub __sourceid2specfilename {
    my %sourceid2spec;
    my $dbh=&Test::Repocop::SQL::connect_db("$repocop_test_dbdir/rpm.db");
    my @row;
    my $sth=$dbh->prepare('SELECT PKGID, SRCFILENAME FROM SRCRPM_FILES WHERE SRCFILEFLAG = 32') or die $dbh->errstr;
    $sth->execute() or die $dbh->errstr;
    while (@row=$sth->fetchrow_array()) {
	$sourceid2spec{$row[0]}=$row[1];
    }
    &Test::Repocop::SQL::disconnect_db($dbh);
    return \%sourceid2spec;
}

sub _add_packager {
    my ($key,$packager)=@_;
    $packager=$1 if $packager=~/<(.+)>/;
    $packager=~s/\@.+$// if $packager;
    $packager=~s/\s.+$// if $packager;
    $PKGID2NICK{$key}=$packager if $packager;
}

sub _load {
    my $dbh=&Test::Repocop::SQL::connect_db("$repocop_test_dbdir/rpm.db");
    my @row;
    my $sth=$dbh->prepare('SELECT PKGID, NAME, VERSION, RELEASE, ARCH, PACKAGER, SOURCEID FROM RPM') or die $dbh->errstr;
    $sth->execute() or die $dbh->errstr;
    while (@row=$sth->fetchrow_array()) {
	&__add_bin_row(@row);
    }
    $sth=$dbh->prepare('SELECT PKGID, NAME, VERSION, RELEASE, PACKAGER FROM SRCRPM') or die $dbh->errstr;
    $sth->execute() or die $dbh->errstr;
    while (@row=$sth->fetchrow_array()) {
	&__add_src_row(@row);
    }
    &Test::Repocop::SQL::disconnect_db($dbh);
}

sub __add_bin_row {
    my @row=@_;
    $PKGID2NAME{$row[0]}=$row[1];
    $PKGID2VERSION{$row[0]}=$row[2];
    $PKGID2RELEASE{$row[0]}=$row[3];
    $PKGID2ARCH{$row[0]}=$row[4];
    $PKGID2PACKAGER{$row[0]}=$row[5];
    $PKGID2SOURCEID{$row[0]}=$row[6];
    &_add_packager($row[0],$row[5]);
}

sub __add_src_row {
    my @row=@_;
    $PKGID2NAME{$row[0]}=$row[1];
    $PKGID2VERSION{$row[0]}=$row[2];
    $PKGID2RELEASE{$row[0]}=$row[3];
    # srcrpm table has arch column, but it is either noarch or reflects the arch srpm was built.
    $PKGID2ARCH{$row[0]}='src';
    $PKGID2PACKAGER{$row[0]}=$row[4];
    # for source rpms their source id is their pkgid.
    $PKGID2SOURCEID{$row[0]}=$row[0];
    &_add_packager($row[0],$row[4]);
}


1;

=head1	NAME

Test::Repocop::Metadata -   for repocop

=head1	DESCRIPTION

TODO

=head1	AUTHOR

Written by Igor Vlasenko <viy@altlinux.org>.

=head1	ACKNOWLEGEMENTS

To Alexey Torbin <at@altlinux.org>, whose qa-robot package
had a strong influence on repocop.

=head1	COPYING

Copyright (c) 2008-2022 Igor Vlasenko, ALT Linux Team.

This is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

=cut

