package Test::Repocop::PkgId;

use strict;
use warnings;
use File::Basename;

our $VERSION=0.001;

sub get_pkgid_from_rpm_file {
    my ($rpmarg)=@_;
    my $repocop_pkg_key=basename($rpmarg);
    $repocop_pkg_key=~s/\.rpm$//;
    return $repocop_pkg_key;
}

sub is_pkgid {
    return $_[0]=~m/\.(?:src|[a-z_0-9]+)(:[\d\.]+)?$/;
}

sub pkgid_is_source {
    return $_[0]=~m/\.src(:[\d\.]+)?$/;
}

sub pkgid2reportid {
    #return $_[0]=~/:.*$/r;
    return $_[0];
}

sub srpmfile2srcid {
    local $_=shift;
    s!^.*/!!;
    s/\.rpm$//;
    return $_;
}

sub srcid2srpmfile {
    return $_[0].'.rpm';
}

# public static members
sub specfile2srcid {
    my $id=basename(shift());
    #$id=~s!^.*/!!;
    $id=~s/\.spec$//;
    $id.='.src' if $id!~/\.src$/;
    return $id;
}

sub nvr2srcid {
    return $_[0].'.src';
}

# public static members
sub patchname2srcid {
    my $id=basename(shift());
    $id=~s/(?:-(?:digest|cummulative|safe))?\.(?:patch|diff)$//;
    $id.='.src' if $id!~/\.src$/;
    return $id;
}

sub srcid2patchprefix {
    my $sourceid=shift;
    $sourceid=~s/\.src$//;
    return $sourceid;
}

1;

=head1	NAME

Test::Repocop::PkgId - PkgId subroutines for repocop.

=head1	DESCRIPTION

PkgId subroutines for repocop.
The interface is subject to change without notice.

=head1	AUTHOR

Written by Igor Vlasenko <viy@altlinux.org>.

=head1	ACKNOWLEGEMENTS

To Alexey Torbin <at@altlinux.org>, whose qa-robot package
had a strong influence on repocop.

=head1	COPYING

Copyright (c) 2019-2020 Igor Vlasenko, ALT Linux Team.

This is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

=cut

