package Test::Repocop::RPMdb;
use strict;
use DBI;
use Test::Repocop::SQL;
use Test::Repocop::PkgId;

my $dbh;

my $sth_insert_files;
my $sth_insert_srcfiles;
my $sth_insert_metadata;
my $sth_insert_srcmetadata;
my $sth_insert_triggers;

sub connect_rpm_db {
    my ($dbfile)=@_;
    $dbh=&Test::Repocop::SQL::connect_db($dbfile);
    $sth_insert_files=$dbh->prepare('INSERT INTO RPM_FILES VALUES(?,?,?,?,?,?,?,?,?)') or die $dbh->errstr;
    $sth_insert_srcfiles=$dbh->prepare('INSERT INTO SRCRPM_FILES VALUES(?,?,?,?,?,?,?,?,?)') or die $dbh->errstr;
    $sth_insert_metadata=$dbh->prepare('INSERT INTO RPM VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?)') or die $dbh->errstr;
    $sth_insert_srcmetadata=$dbh->prepare('INSERT INTO SRCRPM VALUES(?,?,?,?,?,?,?,?,?,?,?,?)') or die $dbh->errstr;
    $sth_insert_triggers=$dbh->prepare('INSERT INTO RPM_TRIGGERS VALUES(?,?,?,?,?,?,?)') or die $dbh->errstr;
}

sub done_rpm_db {
    return unless $dbh;
    $sth_insert_files->finish();
    $sth_insert_srcfiles->finish();
    $sth_insert_metadata->finish();
    $sth_insert_srcmetadata->finish();
    $sth_insert_triggers->finish();
    &Test::Repocop::SQL::disconnect_db($dbh);
    $dbh=undef;
}

#my $pkg_counter=0;
sub add_rpm_metadata {
    my ($rhref,$key)=@_;
#http://refspecs.freestandards.org/LSB_3.1.0/LSB-Core-generic/LSB-Core-generic/pkgformat.html
    my $filenamesref=$rhref->filenames();
    if ($filenamesref) {
	my @filename=@{$filenamesref};
	my @fileflag=@{$rhref->{FILEFLAGS}};
	my @filemode=@{$rhref->{FILEMODES}};
	my @filesize=@{$rhref->{FILESIZES}};
	my @fileuser=@{$rhref->{FILEUSERNAME}};
	my @filegroup=@{$rhref->{FILEGROUPNAME}};
	my @filelinkto=@{$rhref->{FILELINKTOS}};
	my @filelang=@{$rhref->{FILELANGS}};

	for (my $i=0;$i<scalar @filename;$i++) {
	    if ($rhref->is_source()) {
		$sth_insert_srcfiles->execute($key,$filename[$i],$fileflag[$i],$filemode[$i],$filesize[$i],$fileuser[$i],$filegroup[$i],$filelinkto[$i],$filelang[$i]) or die $dbh->errstr;
	    } else {
		$sth_insert_files->execute($key,$filename[$i],$fileflag[$i],$filemode[$i],$filesize[$i],$fileuser[$i],$filegroup[$i],$filelinkto[$i],$filelang[$i]) or die $dbh->errstr;
	    #$sth_insert_files->finish or die $dbh->errstr;
	    }
	}
    }

    my %sourcemod=map {$_=>1} qw/REQUIRE CONFLICT/;

    foreach my $tagclass ('PROVIDE','REQUIRE','CONFLICT','OBSOLETE') {
	next unless $rhref->{$tagclass.'NAME'};
	my @name=@{$rhref->{$tagclass.'NAME'}};
	my @flag=@{$rhref->{$tagclass.'FLAGS'}};
	my @version=@{$rhref->{$tagclass.'VERSION'}};
	for (my $i=0;$i<scalar(@name);$i++) {
	    # use RPM_BUILD* for source rpms
	    my $mod = $rhref->is_source() && $sourcemod{$tagclass} ? 'BUILD' : '';
	    my $sqlquery="INSERT INTO RPM_$mod${tagclass}S VALUES (?,?,?,?)";
	    my $sth=$dbh->prepare($sqlquery) or die $dbh->errstr;
	    $sth->execute($key,$name[$i],$flag[$i],$version[$i]) or die $dbh->errstr;
	    #$sth->finish() or die $dbh->errstr;
	}
    }

    foreach my $tagclass ('PROVIDE','OBSOLETE') {
	next unless $rhref->{$tagclass.'S'};
	my @name=@{$rhref->{$tagclass.'S'}};
	my @flag=@{$rhref->{$tagclass.'FLAGS'}};
	my @version=@{$rhref->{$tagclass.'VERSION'}};
	for (my $i=0;$i<scalar(@name);$i++) {
	    # use RPM_BUILD* for source rpms
	    my $mod = $rhref->is_source() && $sourcemod{$tagclass} ? 'BUILD' : '';
	    my $sqlquery="INSERT INTO RPM_$mod${tagclass}S VALUES (?,?,?,?)";
	    my $sth=$dbh->prepare($sqlquery) or die $dbh->errstr;
	    $sth->execute($key,$name[$i],$flag[$i],$version[$i]) or die $dbh->errstr;
	    #$sth->finish() or die $dbh->errstr;
	}
    }

    foreach my $tagclass ('PREIN','POSTIN','PREUN','POSTUN') {
	# note: for non-trivial progs @script=0 but @prog=1
	next unless $rhref->{$tagclass.'PROG'};
        my @script;
        @script=@{$rhref->{$tagclass}} if $rhref->{$tagclass};
	die "impossible: multiple $tagclass scripts in rpm file $key; needs SQL schema modification" if @script>1;
	my @prog=@{$rhref->{$tagclass.'PROG'}};
	my $prog=shift @prog;
	# note: for %post -p "%post_service %name" @prog>1
	# we put it into script column
	unshift @script, join (' ',@prog) if @prog>0;
	my $script=join ('\n',@script);

	my $sqlquery="INSERT INTO RPM_SCRIPTS_${tagclass} VALUES (?,?,?)";
	my $sth=$dbh->prepare($sqlquery) or die $dbh->errstr;
	$sth->execute($key,$script,$prog) or die $dbh->errstr;
	#$sth->finish() or die $dbh->errstr;
    }

    my @rpm_metadata=($key);
    foreach my $tag ('NAME','VERSION','RELEASE','ARCH','EPOCH','PACKAGER','LICENSE','GROUP','URL','DISTRIBUTION','VENDOR') {
	my $val = $rhref->{$tag};
	push @rpm_metadata, $val;
    }
    if ($rhref->is_source()) {
	$sth_insert_srcmetadata->execute(@rpm_metadata) or die $dbh->errstr;
    } else {
	$sth_insert_metadata->execute(@rpm_metadata, &Test::Repocop::PkgId::srpmfile2srcid($rhref->{'SOURCERPM'})) or die $dbh->errstr;
    }

    if ($rhref->{TRIGGERNAME}) {
	my @triggername=@{$rhref->{TRIGGERNAME}};
	my @triggerflag=@{$rhref->{TRIGGERFLAGS}};
	my @triggerversion=@{$rhref->{TRIGGERVERSION}};
	# not used
	#my @triggerconds=@{$rhref->{TRIGGERCONDS}};
	my @triggerindex=@{$rhref->{TRIGGERINDEX}};
	my @triggerscriptprog=@{$rhref->{TRIGGERSCRIPTPROG}};
	my @triggerscript=@{$rhref->{TRIGGERSCRIPTS}};

	for (my $i=0;$i<scalar @triggername;$i++) {
	    my $idx=$triggerindex[$i];
	    $sth_insert_triggers->execute($key,$triggername[$i],$triggerflag[$i],$triggerversion[$i],$triggerscriptprog[$idx],$triggerscript[$idx],$i) or die $dbh->errstr;
	    #$sth_insert_triggers->finish or die $dbh->errstr;
	}
    }

    #$dbh->commit() if ++$pkg_counter % 100 == 0; # every 100 packages
    $dbh->commit() or die $dbh->errstr;;
}

sub get_all_rpm_db_ids {
    die "Oops! not connected to rpm DB" unless $dbh;
    my $select_rpm=$dbh->selectcol_arrayref('SELECT PKGID FROM RPM') or die $dbh->errstr;
    my $select_srpm=$dbh->selectcol_arrayref('SELECT PKGID FROM SRCRPM') or die $dbh->errstr;
    my %rpmids;
    map {$rpmids{$_}=1} @$select_rpm, @$select_srpm;
    return \%rpmids;
}

# only used in Options, so we require separate $dbh
sub get_rpm_db_ids_by_srpm_name {
    my ($dbh, $namesref)=@_;
    die "Oops! not connected to rpm DB" unless $dbh;
    die "Oops! not a ref: $namesref" if not ref $namesref;
    return if not @$namesref;
    $dbh->do('CREATE TEMPORARY TABLE src_names_'.$$.' (TMPSRCNAME TEXT)');
    my $sth_insert_names=$dbh->prepare('INSERT INTO src_names_'.$$.' VALUES(?)') or die $dbh->errstr;
    map {$sth_insert_names->execute($_)} @$namesref;
    $sth_insert_names->finish();
    $dbh->do('CREATE TEMPORARY TABLE src_ids_by_srpm_'.$$.' (TMPSRCID TEXT)');
    $dbh->do('INSERT INTO src_ids_by_srpm_'.$$.' SELECT PKGID FROM SRCRPM WHERE NAME IN (SELECT * FROM src_names_'.$$.')');
    $dbh->do('DROP TABLE src_names_'.$$);

    my $select_rpm=$dbh->selectcol_arrayref('SELECT PKGID FROM RPM WHERE SOURCEID IN (SELECT * FROM src_ids_by_srpm_'.$$.')') or die $dbh->errstr;
    my $select_srpm=$dbh->selectcol_arrayref('SELECT * FROM src_ids_by_srpm_'.$$) or die $dbh->errstr;

    $dbh->do('DROP TABLE src_ids_by_srpm_'.$$);
    return $select_srpm, $select_rpm;
}



1;

=head1	NAME

Test::Repocop::RPMdb - rpm.db abstraction layer for repocop

=head1	DESCRIPTION

Test::Repocop::RPMdb - hides rpm.db sql scheme from the rest of repocop

=head1	AUTHOR

Written by Igor Vlasenko <viy@altlinux.org>.

=head1	ACKNOWLEGEMENTS

To Alexey Torbin <at@altlinux.org>, whose qa-robot package
had a strong influence on repocop.

=head1	COPYING

Copyright (c) 2008-2022 Igor Vlasenko, ALT Linux Team.

This is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

=cut

