package Test::Repocop::TestDB::Backend::Filesystem;

use strict;
use warnings;

use File::Path qw(make_path remove_tree);
use File::Basename;
use Test::Repocop::Workdir;
use Test::Repocop::Log;
use Test::Repocop::TestDB::Filter; # just for Repocop::arg :(

# TODO: made it a set of plug-ins
#use Test::Repocop::TestDB::Backend::Filesystem::CRCPrefix;
use Test::Repocop::TestDB::Backend::Filesystem::SumPrefix;


our $repocop_test_outputdirname='testdb';
our $repocop_test_backend='fsprefix';
our $repocop_test_backend_prefix_width=1;
my $repocop_test_backend_prefix=\&fsprefix;

sub new {
    my $class=shift;
    my $self = {
    };
    bless $self, $class;
    &__detect_backend();
    return $self;
}

### internal hack
sub __detect_backend {
    if (-e '/usr/libexec/repocop/backends/fsprefix.bin') {
	$repocop_test_backend='fsprefix.bin';
    }
    my ($alttestdb)=glob($repocop_workdir.'/testdb-fs-sum[bx]-[1-9]');
    if ($alttestdb and $alttestdb =~ m!/(testdb-fs-sum([bx])-([1-9]))$!) {
	$repocop_test_outputdirname=$1;
	my $sumformat = $2;
	$repocop_test_backend_prefix_width=$3;
	if ($sumformat eq 'b') {
	    $repocop_test_backend='fssumbprefix.bin';
	    $repocop_test_backend_prefix=&Test::Repocop::TestDB::Backend::Filesystem::SumPrefix::get_fs_sumb_prefix_func($repocop_test_backend_prefix_width);
	} elsif ($sumformat eq 'x') {
	    $repocop_test_backend='fssumxprefix.bin';
	    $repocop_test_backend_prefix=&Test::Repocop::TestDB::Backend::Filesystem::SumPrefix::get_fs_sumx_prefix_func($repocop_test_backend_prefix_width);
	}
    }
    die 'FATAL: repocop backend ',$repocop_test_backend, ' not found'
	if ! -e '/usr/libexec/repocop/backends/'.$repocop_test_backend;
}

sub fsprefix {
    my $repocop_pkg_key = shift;
    return substr($repocop_pkg_key,0,$repocop_test_backend_prefix_width);
}

sub setup_environment {
    my $self=shift;
    my $repocop_test_outputdir="$repocop_workdir/$repocop_test_outputdirname";
    make_path($repocop_test_outputdir);
    # for repocop-test-* utils
    # for example, used in fsprefix.bin
    $ENV{'REPOCOP_TEST_BACKEND'}=$repocop_test_backend;
    $ENV{'REPOCOP_TEST_OUTPUTDIR'}=$repocop_test_outputdir;
    $ENV{'REPOCOP_TEST_BACKEND_PREFIX_WIDTH'}=$repocop_test_backend_prefix_width;
    # set PATH
    $ENV{'PATH'}='/usr/libexec/repocop/backends/'.$repocop_test_backend.':'.$ENV{'PATH'};
}

sub environment_for_export {
    return qw/
	REPOCOP_TEST_OUTPUTDIR
	REPOCOP_TEST_BACKEND
	REPOCOP_TEST_BACKEND_PREFIX_WIDTH
	PATH
	/;
}

sub remove_test {
    my $self=shift;
    return unless @_;
    my @findclause;
    foreach my $testname (@_) {
	push @findclause, '-name',$testname,'-or','-name',$testname.'+++*', '-or';
    }
    pop @findclause if @findclause;
    system ('find',"$repocop_workdir/$repocop_test_outputdirname",'-type','f','(',@findclause,')','-delete') == 0 || die "error: $!";
}

# static internal
sub __get_touchfile {
    my ($repocop_pkg_key,$testname)=@_;
    my $prefix=&$repocop_test_backend_prefix($repocop_pkg_key);
    my $path="$repocop_workdir/$repocop_test_outputdirname/$prefix/$repocop_pkg_key";
    $path.="/$testname" if defined $testname;
    return $path;
}

sub is_test_performed {
    my $self=shift;
    my ($repocop_pkg_key,$testname)=@_;
    return -e &__get_touchfile($repocop_pkg_key,$testname);
}

sub get_test_mtime {
    my $self=shift;
    my ($repocop_pkg_key,$testname)=@_;
    return -M &__get_touchfile($repocop_pkg_key,$testname);
}

sub set_test_mtime {
    my $self=shift;
    my ($repocop_pkg_key,$testname)=@_;
    my $repocop_test_result=&__get_touchfile($repocop_pkg_key,$testname);
    make_path(dirname($repocop_test_result));
    system('touch',$repocop_test_result)==0 or die "touch $repocop_test_result: $!";
}

sub clear_test_result_and_mtime {
    my $self=shift;
    my ($repocop_pkg_key,$testname)=@_;
    my $repocop_test_result=&__get_touchfile($repocop_pkg_key,$testname);
    unlink <$repocop_test_result*>;
}

sub delete_pkg {
    my $self=shift;
    my ($repocop_pkg_key)=@_;
    return unless $repocop_pkg_key;
    remove_tree(&__get_touchfile($repocop_pkg_key));
}

sub get_pkg_test_status_iterator {
    return Test::Repocop::TestDB::Backend::Filesystem::TestIterator->new();
}

sub get_pkg_test_status_result_iterator {
    return Test::Repocop::TestDB::Backend::Filesystem::TestIterator->new();
}

sub purge {
    my $self=shift;
    my $testPurger=Test::Repocop::TestDB::Backend::Filesystem::__PackageIterator->new();
    while ($_=$testPurger->iterate_pkg()) {
	next if &Repocop::arg::skip_pkg($_);
	&repocop_note("Testdb: deleting: $_");
	$self->delete_pkg($_);
    }
}

package Test::Repocop::TestDB::Backend::Filesystem::TestIterator;

use strict;
use warnings;

use Test::Repocop::TestDB::Filter; # just for Repocop::arg :(
use Test::Repocop::Workdir;
use File::Path qw(make_path remove_tree);
use File::Basename;

# import variables
*repocop_test_outputdirname=*Test::Repocop::TestDB::Backend::Filesystem::repocop_test_outputdirname;
*repocop_test_backend_prefix_width=*Test::Repocop::TestDB::Backend::Filesystem::repocop_test_backend_prefix_width;

sub new {
    my $class=shift;
    my $fd;
    my $self = {
	FD => $fd,
	CUR_TEST_FILENAME => '',
	FILTER_NICK=> 0,
    };
    bless $self, $class;
    open ($self->{FD}, " find $repocop_workdir/$repocop_test_outputdirname/ -type f -name '*+++*' |");
    return $self;
}

sub DESTROY {
    my $self=shift;
    close ($self->{FD});
}

sub iterate3 {
    my $self=shift;
    local $_;
    $_=readline $self->{FD};
    return unless $_;
    chomp $_;
    $self->{CUR_TEST_FILENAME}=$_;
    if ($repocop_test_backend_prefix_width) {
	s!^$repocop_workdir/$repocop_test_outputdirname/[^/]+/?!!;
    } else {
	s!^$repocop_workdir/$repocop_test_outputdirname/?!!;
    }
    #print 'iterate3: ',$_,' raw: ',$self->{CUR_TEST_FILENAME},"\n";
    my ($rpmkey,$teststatus)=split '/';
    my ($test,$status)=split '\+\+\+', $teststatus;
    return ($rpmkey,$test,$status);
}

sub iterate3_filter_by_test_status {
    my $self=shift;
    my ($rpm,$test,$status);
    do {
	($rpm,$test,$status)=$self->iterate3();
    } while $rpm && ( &Repocop::arg::skip_test($test) || &Repocop::arg::skip_status($status) );
    return unless $rpm;
    return ($rpm,$test,$status);
}

sub iterate3_filtered {
    my $self=shift;
    my ($rpm,$test,$status);
    do {
	($rpm,$test,$status)=$self->iterate3();
    } while $rpm && ( &Repocop::arg::skip_test($test) || &Repocop::arg::skip_pkg($rpm) || &Repocop::arg::skip_status($status) );
    return unless $rpm;
    return ($rpm,$test,$status);
}

sub iterate4 {
    my $self=shift;
    my ($rpm,$test,$status)=$self->iterate3();
    return unless $rpm;
    my $result=$self->__result();
    return ($rpm,$test,$status,$result);
}

sub iterate4_filtered {
    my $self=shift;
    my ($rpm,$test,$status)=$self->iterate3_filtered();
    return unless $rpm;
    my $result=$self->__result();
    return ($rpm,$test,$status,$result);
}

sub __result {
    my $self=shift;
    return &_read_file($self->{CUR_TEST_FILENAME});
}


sub _read_file {
    my $filename=shift;
    return "" unless -s $filename;
    open my $fh, $filename or die $!;
    local $/; # enable localized slurp mode
    my $content = <$fh>;
    close $fh;
    return $content;
}

package Test::Repocop::TestDB::Backend::Filesystem::__PackageIterator;

use strict;
use warnings;
use Test::Repocop::Workdir;

# import variables
*repocop_test_outputdirname=*Test::Repocop::TestDB::Backend::Filesystem::repocop_test_outputdirname;
*repocop_test_backend_prefix_width=*Test::Repocop::TestDB::Backend::Filesystem::repocop_test_backend_prefix_width;

sub new {
    my $class=shift;
    my $fd;
    my $self = {
	FD => $fd,
    };
    bless $self, $class;
    open ($self->{FD}, "find $repocop_workdir/$repocop_test_outputdirname/".('?'x$repocop_test_backend_prefix_width)." -type d |");
    return $self;
}

sub DESTROY {
    my $self=shift;
    close ($self->{FD});
}

sub iterate_pkg {
    my $self=shift;
    while (1) {
	local $_;
	$_=readline $self->{FD};
	return unless $_;
	chomp $_;
	#my $filename=$_;
	if ($repocop_test_backend_prefix_width) {
	    s!^$repocop_workdir/$repocop_test_outputdirname/[^/]+/?!!;
	} else {
	    s!^$repocop_workdir/$repocop_test_outputdirname/?!!;
	}
	next unless $_;
	return $_;
    }
}

1;

