package Test::Repocop::TestDB::Backend::Filesystem::SumPrefix;

use strict;
use warnings;
use integer;

my @BASE64URI=split(//,'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_');

sub sum {
    my $instring=shift;
    my $sum=0;
    map {$sum+=ord($_)} split (//,$instring);
    return $sum;
}

sub get_fs_sumb_prefix_func {
    my $num=shift;
    #warn 'DEBUG: get_fs_sumb_prefix_func: num=',$num;
    return \&fssumbprefix1 if $num==1;
    return \&fssumbprefix2 if $num==2;
    my $power=1;
    for (my $i=0; $i<$num; $i++) {
	$power *= 64;
    }
    my $sumfunc = sub {
	my $repocop_pkg_key = shift;
	my $sum = &sum($repocop_pkg_key) % $power;
	my $retval='';
	my $curpower=$power / 64;
	for (my $i=0; $i<$num; $i++) {
	    $retval .= $BASE64URI[$sum / $curpower];
	    $sum %= $curpower;
	    $curpower = $curpower / 64;
	}
	return $retval;
    };
    return $sumfunc;
}

sub fssumbprefix1 {
    my $repocop_pkg_key = shift;
    my $sum = &sum($repocop_pkg_key) % 64; # 1 first base symbols
    return $BASE64URI[$sum];
}

sub fssumbprefix2 {
    my $repocop_pkg_key = shift;
    my $sum = &sum($repocop_pkg_key) % 4096; # 2 first base symbols
    return $BASE64URI[$sum / 64].$BASE64URI[$sum % 64];
}

# - fssumxprefix --

# lambda func
sub get_fs_sumx_prefix_func {
    my $num=shift;
    return \&fssumxprefix1 if $num==1;
    return \&fssumxprefix2 if $num==2;
    return \&fssumxprefix3 if $num==3;
    return \&fssumxprefix4 if $num==4;
    my $power=1;
    for (my $i=0; $i<$num; $i++) {
	$power *= 16;
    }
    my $format='%0'.$num.'x';
    return sub {
	my $repocop_pkg_key = shift;
	my $sum = &sum($repocop_pkg_key) % $power;
	return sprintf( $format , $sum );
    }
}

sub fssumxprefix1 {
    my $repocop_pkg_key = shift;
    my $sum = &sum($repocop_pkg_key) % 16; # 1 first hex digits
    return sprintf( '%01x', $sum );
}

sub fssumxprefix2 {
    my $repocop_pkg_key = shift;
    my $sum = &sum($repocop_pkg_key) % 256; # 2 first hex digits
    return sprintf( '%02x', $sum );
}

sub fssumxprefix3 {
    my $repocop_pkg_key = shift;
    my $sum = &sum($repocop_pkg_key) % 4096; # 3 first hex digits
    return sprintf( '%03x', $sum );
}

sub fssumxprefix4 {
    my $repocop_pkg_key = shift;
    my $sum = &sum($repocop_pkg_key) % 65536 ; # 3 first hex digits
    return sprintf( '%04x', $sum );
}

1;
