package Test::Repocop::Workdir;
use strict;
use warnings;
use Cwd();
use File::Lock::ParentLock;
require Exporter;
our @ISA = qw(Exporter);

our @EXPORT = qw($repocop_workdir $repocop_metadatadir $repocop_test_dbdir);
our @EXPORT_OK = qw($repocop_testcachedir);
push @EXPORT_OK, @EXPORT;

our $repocop_workdir="$ENV{HOME}/.repocop";
# TODO: REPOCOP_CACHEDIR deprecated
$repocop_workdir=$ENV{'REPOCOP_CACHEDIR'} if $ENV{'REPOCOP_CACHEDIR'};
$repocop_workdir=$ENV{'REPOCOP_WORKDIR'} if $ENV{'REPOCOP_WORKDIR'};
our ($repocop_metadatadir,$repocop_test_dbdir);
our ($repocop_testcachedir);

my $lockname = '.lock';
our $lockfile;

sub set_workdir_variables {
    $repocop_workdir=Cwd::abs_path($repocop_workdir) unless $repocop_workdir=~m!^/!;
    $repocop_metadatadir="$repocop_workdir/metadata";
    $repocop_test_dbdir="$repocop_workdir/db";
    $repocop_testcachedir="$repocop_workdir/testcache";
    $lockfile="$repocop_workdir/$lockname";
}

sub die_if_nothing_to_report {
    die "repocop workdir $repocop_workdir does not exist or is not a directory!\n" if ( ! -d "$repocop_workdir");
    die "Results of tests are not found in $repocop_workdir - nothing to do. run repocop-run first!\n" if (! -d "$repocop_test_dbdir");
}

sub get_locker {
    return my $locker=File::Lock::ParentLock->new(-lockfile=>$lockfile);
}

1;

=head1	NAME

Test::Repocop::Workdir - common repocop workdir wariables

=head1	SYNOPSIS

B<repocop-*>
=include synopsis.pod.in

=head1	DESCRIPTION

B<repocop-*> processes results of repocop unit tests, created with 
repocop-run command, stored in I<repocop workdir>.
Precise subset of tests can be selected using B<--include>
and B<--exclude> options.

=head1	OPTIONS

=over

=include options-report.pod.in
=include options.pod.in

=back

=head1	AUTHOR

Written by Igor Vlasenko <viy@altlinux.org>.

=head1	COPYING

Copyright (c) 2008-2022 Igor Vlasenko, ALT Linux Team.

This is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

=cut

